import trollius
import urlparse
import ssl
from config import PEER_CLIENT_ID, logger, HTTP_CLIENT_TIMEUOT, EXTERNAL_PORT
import re
import urllib
import bencode
import socket
import struct
import time

re_find_result_code = re.compile('HTTP.*?\s(\d+)')

USER_AGENT = 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.87 Safari/537.36'

if False:
    from LTorrentEngine import LTorrentEngine


def get_peer_from_url(url):
    # type: (str) -> (str, int, bool) # address, port, is_https
    p_url = urlparse.urlparse(url)
    if p_url.netloc.find(':') == -1:
        return p_url.netloc, 80 if p_url.scheme == 'http' else 443, p_url.scheme == 'https'
    else:
        addr, port = p_url.netloc.split(':')
        return addr, int(port), p_url.scheme == 'https'


class AsyncHTTPAnnounce(trollius.Protocol):
    def __init__(self, engine, url):
        # type: (LTorrentEngine) -> None
        self.engine = engine
        self.url = url
        self.peer_info = get_peer_from_url(url)
        self.transport = None
        p_url = urlparse.urlparse(url)
        self.path = p_url.path if p_url.path else '/'
        self.data = ''
        self.status = 0
        self.headers = {}
        self.result_code = None
        self.content_length = 0
        self.timeout_timer = None  # type: trollius.events.TimerHandle
        self.loop = trollius.get_event_loop()  # type: trollius.BaseEventLoop

    def debug(self, message):
        logger.debug('AsyncHTTPAnnounce %s:%s %s' % (self.peer_info[0], self.peer_info[1], message))

    def info(self, message):
        logger.info('AsyncHTTPAnnounce %s:%s %s' % (self.peer_info[0], self.peer_info[1], message))

    def connection_made(self, transport):
        self.transport = transport
        self.info('connection made')
        req = {
            'info_hash': self.engine.cache.torrent.info_hash(),
            'peer_id': PEER_CLIENT_ID,
            'uploaded': self.engine.cache.stat['send_bytes'],
            'downloaded': self.engine.cache.stat['receive_bytes'],
            'left': self.engine.cache.left(),
            'port': EXTERNAL_PORT,
            'compact': 1,
        }

        req = '%s?%s' % (str(self.path), urllib.urlencode(req))
        self.transport.write('GET %s HTTP/1.1\r\nHost: %s\r\nUser-Agent: %s\r\n\r\n' % (req, str(self.peer_info[0]), USER_AGENT))
        self.timeout_timer = self.loop.call_later(HTTP_CLIENT_TIMEUOT, self.timeout)

    def timeout(self):
        self.transport.close()

    def data_received(self, data):
        if self.timeout_timer is not None:
            self.timeout_timer.cancel()
            self.timeout_timer = self.loop.call_later(HTTP_CLIENT_TIMEUOT, self.timeout)
        self.data += data
        if self.status == 0:
            split_pos = self.data.find('\r\n\r\n')
            if split_pos == -1:
                return
            else:
                headers = self.data[:split_pos]
                self.data = self.data[split_pos + 4:]
                self.status = 1
                for i, header in enumerate(headers.split('\r\n')):
                    if i == 0:
                        self.result_code = int(re_find_result_code.findall(header)[0])
                    else:
                        sh = header.strip()
                        if sh:
                            sh_s = sh.find(':')
                            if sh_s != -1:
                                self.headers[sh[:sh_s].strip()] = sh[sh_s + 1:].strip()
                if self.result_code != 200:
                    self.info('bad result code %s' % self.result_code)
                    self.transport.close()
                    return
                if 'Content-Length' in self.headers:
                    self.content_length = int(self.headers['Content-Length'])
        else:
            if len(self.data) >= self.content_length:
                self.transport.close()

    def connection_lost(self, exc):
        if self.timeout_timer is not None:
            self.timeout_timer.cancel()
        self.info('connection lost')
        if self.result_code == 200 and self.content_length == len(self.data):
            announce = bencode.decode(self.data)
            if 'peers' not in announce:
                self.info('Not found peers in announce')
                return
            for i in range(len(announce['peers']) / 6):
                p = announce['peers'][(i * 6):(i + 1) * 6]
                self.engine.a_add_peer((socket.inet_ntoa(p[:4]), struct.unpack('!H', p[4:])[0]), 'http_announce')
            self.info('Announce complete')
            for i in self.engine.announcers:
                if i[0] == self.url:
                    break
            else:
                timeout = min(announce['interval'], announce['min interval'])
                self.engine.announcers.append([self.url, timeout, time.time() + 60])


def test1():
    from torrent import Torrent
    from cache import Cache

    class SimulatedEngine:
        def __init__(self, cache):
            self.cache = cache

        def a_add_peer(self, peer):
            print peer

    t = Torrent(fn='tmp/House.M.D.s01.rus.LostFilm.TV.torrent')
    c = Cache(t)
    engine = SimulatedEngine(c)
    print t.data['announce']
    loop = trollius.get_event_loop()  # type: trollius.BaseEventLoop
    url = t.data['announce']
    host, port, https = get_peer_from_url(url)
    coro = loop.create_connection(lambda: AsyncHTTPAnnounce(engine, url), host, port, None if not https else ssl.create_default_context())
    loop.create_task(coro)
    loop.run_forever()
    loop.close()


def test2():
    from torrent import Torrent
    from http_client import HTTPClient
    t = Torrent(fn='tmp/House.M.D.s01.rus.LostFilm.TV.torrent')
    print t.data['announce']

    req = {
        'info_hash': t.info_hash(),
        'peer_id': PEER_CLIENT_ID,
        'uploaded': 0,
        'downloaded': 0,
        'left': t.full_size,
        'port': EXTERNAL_PORT,
        'compact': 1
    }

    http_client = HTTPClient()
    res = http_client.get_page('%s?%s' % (t.data['announce'], urllib.urlencode(req)))
    print res


if __name__ == '__main__':
    test1()
