import trollius
from config import PEER_CLIENT_ID, logger, EXTERNAL_PORT, EXTERNAL_IP
import struct
import random
import socket
import time

if False:
    from LTorrentEngine import LTorrentEngine


class AsyncUDPAnnounce(trollius.DatagramProtocol):
    def __init__(self, engine, url):
        # type: (LTorrentEngine, str) -> None
        self.url = url
        self.engine = engine
        self.transport = None
        self.status = 0
        self.transaction_id = None
        self.peername = None
        self.connection_id = None

    def debug(self, message):
        logger.debug('AsyncUDPAnnouncer %s:%i %s' % (self.peername[0], self.peername[1], message))

    def info(self, message):
        logger.info('AsyncUDPAnnouncer %s:%i %s' % (self.peername[0], self.peername[1], message))

    def generate_transaction_id(self):
        self.transaction_id = random.randint(0, (1 << 32) - 1)
        return self.transaction_id

    def connection_made(self, transport):
        # type: (trollius.transports.DatagramTransport) -> None
        self.transport = transport
        self.peername = transport.get_extra_info('peername')
        self.info('connection made')
        self.transport.sendto(struct.pack('!QII', 0x41727101980, 0, self.generate_transaction_id()))

    def datagram_received(self, data, addr):
        if self.status == 0:
            if len(data) < 16:
                self.debug('small init packet, close')
                self.transport.close()
                return
            action, transaction_id, self.connection_id = struct.unpack('!IIQ', data)
            if transaction_id != self.transaction_id:
                self.debug('bad transaction id, close')
                self.transport.close()
                return
            self.status = 1
            self.transport.sendto(struct.pack('!QII20s20sQQQI4sIiH',
                                              self.connection_id,
                                              1,  # action          1 // announce
                                              self.generate_transaction_id(),
                                              self.engine.cache.torrent.info_hash(),
                                              PEER_CLIENT_ID,
                                              self.engine.cache.stat['receive_bytes'],
                                              self.engine.cache.left(),
                                              self.engine.cache.stat['send_bytes'],
                                              0,  # 0 // 0: none; 1: completed; 2: started; 3: stopped
                                              EXTERNAL_IP,
                                              0,  # key
                                              -1,  # num_want
                                              EXTERNAL_PORT))
        elif self.status == 1:
            if len(data) < 20:
                self.debug('small announce packet, close')
                self.transport.close()
                return
            announce, transaction_id, timeout, leechers, seeders = struct.unpack('!IIIII', data[:5 * 4])
            if announce != 1:
                self.debug('not announce, close')
                self.transport.close()
                return
            if transaction_id != self.transaction_id:
                self.debug('bad transaction id, close')
                self.transport.close()
                return
            for i in range((len(data) - 20) // 6):
                ip, port = struct.unpack('!4sH', data[i * 6 + 20:i * 6 + 26])
                self.engine.a_add_peer((socket.inet_ntoa(ip), port), 'udp_announce')
            self.info('Announce complete')
            self.transport.close()
            for i in self.engine.announcers:
                if i[0] == self.url:
                    break
            else:
                self.engine.announcers.append([self.url, timeout, time.time() + 60])

    def connection_lost(self, exc):
        logger.debug('Connection lost')


def test1():
    from torrent import Torrent
    from cache import Cache
    from async_http_announce import get_peer_from_url

    class SimulatedEngine:
        def __init__(self, cache):
            self.cache = cache

        def a_add_peer(self, peer):
            print peer

    engine = SimulatedEngine(Cache(Torrent(fn='tmp/[rutor.is]Chelovek_muravey_i_Osa_2018_HDRip_iT_by_Dalemak.torrent')))
    print get_peer_from_url(engine.cache.torrent.data['announce'])
    exit(0)
    loop = trollius.get_event_loop()  # type: trollius.BaseEventLoop
    coro = loop.create_datagram_endpoint(lambda: AsyncUDPAnnounce(engine), remote_addr=('46.148.18.250', 2710))
    loop.create_task(coro)
    loop.run_forever()
    loop.close()


if __name__ == '__main__':
    test1()
