import os
import logging
import logging.handlers
import pickle
import string
import random
import time

__VERSION__ = '0.0.3'

MY_DIR = os.path.dirname(os.path.realpath(__file__))
WORK_DIR = os.path.join(MY_DIR, 'cache')
_config_file = os.path.join(WORK_DIR, 'config.pickle')
_config = None

if os.path.exists(_config_file):
    try:
        _config = pickle.load(open(_config_file, 'rb'))
    except:
        pass


def get_config(param, default_value):
    global _config
    if _config is not None:
        if param in _config:
            return _config[param]
    else:
        _config = {}
    _config[param] = default_value
    save_config()
    return default_value


def save_config():
    with open(_config_file, 'wb') as f:
        pickle.dump(_config, f)


def setup_logger(file_name):
    _logger = logging.getLogger()
    _logger.setLevel(logging.DEBUG)
    file_handler = logging.handlers.RotatingFileHandler(os.path.join(MY_DIR, 'log', file_name + '.log'), maxBytes=1024 * 1024 * 8, backupCount=3)
    formater = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
    file_handler.setFormatter(formater)
    _logger.addHandler(file_handler)
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(formater)
    console_handler.setLevel(logging.INFO)
    # console_handler.setLevel(logging.DEBUG)
    _logger.addHandler(console_handler)
    return _logger


logger = setup_logger('LTorrentEngine')

WEB_SERVER_PORT = get_config('WEB_SERVER_PORT', 8888)

DHT_SERVER = False
DHT_ADDR = get_config('DHT_ADDR', random.getrandbits(160))
DHT_PEER_LIMIT = 512
DHT_K_LIMIT = 16
DHT_PEER_PING_TIME = 60*10
DHT_PEER_LIFE_TIME = 60*25
DHT_REQUEST_TIMEOUT = 20
DHT_SEARCH_THREADS = 64
DHT_INIT_NODES = [
    ("router.utorrent.com", 6881),
    ("grenade.genua.fr", 6880),
    ("dht.transmissionbt.com", 6881),
]
DHT_SEARCH_QUERIES_LIMIT = 8000

PEER_CLIENT_ID = get_config('PEER_CLIENT_ID', '-PP0003-' + ''.join(map(lambda x: random.choice(string.printable[:62]), range(12))))  # '-PP0001-' + '0' * 12
EXTERNAL_PORT = get_config('EXTERNAL_PORT', random.randint(10000, 64534))
DHT_EXTERNAL_PORT = get_config('DHT_EXTERNAL_PORT', random.randint(10000, 64534))

UPNP = False

_cache_time = get_config('cache_time', 0)

_external_ip = None
if time.time() - _cache_time < 60 * 60 * 3:  # cache on 3 hours
    _external_ip = get_config('cache_external_ip', None)
    UPNP = get_config('cache_upnp', False)
    logger.info('Get external IP from cache %s' % _external_ip)

if _external_ip is None:
    try:
        from upnp import get_external_ip

        _external_ip = get_external_ip()
        logger.info('Detect external IP %s (UPNP)' % _external_ip)
        UPNP = True
    except:
        try:
            import urllib2

            _external_ip = urllib2.urlopen('https://api.ipify.org/', timeout=2).read()

            logging.info('Detect external IP %s (WEB API)' % _external_ip)
        except:
            _external_ip = '192.168.0.10'
            logging.info('Use fake external ip %s' % _external_ip)
    _config['cache_time'] = time.time()
    _config['cache_external_ip'] = _external_ip
    _config['cache_upnp'] = UPNP
    save_config()

EXTERNAL_IP = _external_ip

MAX_CACHE_FILE_SIZE = 512 * pow(1024, 2)

WEB_SERVER_SEND_BUF = 8 * 1024

CACHE_TORRENT_NAME = 'cache.torrent'
CACHE_FILE_INDEX_NAME = 'file_index'
CACHE_PIECES_NAME = 'cache_%i.pieces'
CACHE_BITFIELD_NAME = 'bitfield'
CACHE_STAT_UPDATE_TIMEOUT = 5

HTTP_CLIENT_TIMEUOT = 3

PEER_SERVER_TIMEOUT = 15
PEER_SERVER_CONN_LIMIT = 256
PEER_MAX_SIMILAR_CONN = 16
PEER_FORCED_RECONNECT = 30

PEER_REQUEST_LENGTH = pow(2, 14)
SLOW_PEER_TIMEOUT = 5

PACKET_HANDSHAKE_LEN = 68

CLIENT_NAME = 'LTorrentEngine %s' % __VERSION__
