import math
import os.path


class FileBitMap:
    def __init__(self, fn, length):
        self.fn = fn
        self.length = length
        if os.path.exists(fn):
            self.file = open(self.fn, 'rb+')
        else:
            self.file = open(self.fn, 'wb+')
            self.file.truncate(math.ceil(float(self.length) / 8))

    def __getitem__(self, item):
        if not isinstance(item, (int, long)):
            raise Exception('item not int')
        if item >= self.length:
            raise Exception('item out of range')
        byte_pos = item // 8
        mask = 1 << (item % 8)
        self.file.seek(byte_pos)
        data = self.file.read(1)
        return (ord(data) & mask) > 0

    def __setitem__(self, item, value):
        byte_pos = item // 8
        mask = 1 << (item % 8)
        self.file.seek(byte_pos)
        data = ord(self.file.read(1))
        data &= ~mask
        if value:
            data |= mask
        self.file.seek(byte_pos)
        self.file.write(chr(data))

    def __iter__(self):
        for i in range(self.length):
            yield self.__getitem__(i)

    def bytes(self):
        self.file.seek(0)
        return self.file.read()


def test1():
    fbm = FileBitMap('test.bin', 20)
    fbm[0] = True
    fbm[2] = True
    fbm[13] = True
    fbm[2] = False
    for i in fbm:
        print fbm[i]


if __name__ == '__main__':
    test1()
