import hashlib
import bencode
import math
from config import PEER_REQUEST_LENGTH


class Torrent:
    def __init__(self, fn=None, raw_data=None):
        if fn is not None:
            with open(fn, 'rb') as f:
                self.raw_data = f.read()
        elif raw_data is not None:
            self.raw_data = raw_data
        else:
            raise Exception('Error in Torrent init fixme')
        self._info_hash = None
        self.data = bencode.decode(self.raw_data)  # type: dict
        self.multifile = 'files' in self.data['info']
        self.piece_length = self.data['info']['piece length']
        self.full_size = self._full_size()
        self.piece_count = int(math.ceil(float(self.full_size) / self.piece_length))

    def get_piece_size(self, piece_index):
        if piece_index < 0:
            piece_index = self.piece_count + piece_index
        ps = self.piece_length
        if piece_index == self.piece_count - 1:
            ps = self.full_size % self.piece_length
            if ps == 0:
                ps = self.piece_length
        return ps

    def get_part_size(self, piece_index, part_index):
        piece_size = self.get_piece_size(piece_index)
        offset = PEER_REQUEST_LENGTH * part_index
        tail = piece_size - offset
        part_size = tail if tail < PEER_REQUEST_LENGTH else PEER_REQUEST_LENGTH  # last piece
        return part_size



    def get_files(self):
        if not self.multifile:
            return self.data['info']['name'],
        else:
            return tuple((i['path'][-1] for i in self.data['info']['files']))

    def _full_size(self):
        if self.multifile:
            return sum((i['length'] for i in self.data['info']['files']))
        return self.data['info']['length']

    def info_hash(self):
        if self._info_hash is None:
            h = hashlib.sha1(bencode.encode(self.data['info']))
            self._info_hash = h.digest()
        return self._info_hash

    def get_piece_hash(self, piece_index):
        return self.data['info']['pieces'][piece_index * 20:(piece_index + 1) * 20]


def test1():
    t = Torrent('Star Trek Discovery 1 - LostFilm.TV.torrent')
    print t.get_files()
    t = Torrent('House.M.D.s01.rus.LostFilm.TV.torrent')
    print t.get_files()[3]


def test2():
    f_index = 4
    t = Torrent('House.M.D.s01.rus.LostFilm.TV.torrent')

    f_size = t.data['info']['files'][f_index]['length']
    skip_size = sum([t.data['info']['files'][i]['length'] for i in range(f_index)])
    first_piece = skip_size // t.piece_length
    first_offset = skip_size % t.piece_length
    a_size = f_size - (t.piece_length - first_offset)
    last_piece = first_piece + int(math.ceil(float(a_size) / t.piece_length))
    last_size = a_size % t.piece_length
    print first_piece, first_offset, last_piece, last_size

    print (last_piece - first_piece) * t.piece_length + last_size - first_offset, f_size
    print (last_piece - first_piece) * t.piece_length + last_size - first_offset == f_size


def test3():
    from config import EXTERNAL_PORT, PEER_CLIENT_ID
    from http_client import HTTPClient
    import urllib
    import trollius
    import time
    t = Torrent(raw_data=open('tmp/Ray.Donovan.s06.HD1080p.WEBRip.Rus.Eng.BaibaKo.tv.torrent', 'rb').read())
    announce_url = t.data['announce']
    print announce_url
    print 'announce-list' in t.data
    print t.data
    req = {
        'info_hash': t.info_hash(),
        'peer_id': PEER_CLIENT_ID,
        'uploaded': 0,
        'downloaded': 0,
        'left': t.full_size,
        'port': EXTERNAL_PORT,
        'compact': 1
    }
    http_client = HTTPClient()
    loop = trollius.get_event_loop()
    f1 = loop.run_in_executor(None, time.sleep, 20)
    print trollius.wait([f1])
    # loop.run_forever()
    # res = http_client.get_page('%s?%s' % (announce_url, urllib.urlencode(req)))
    # print res


if __name__ == '__main__':
    test3()
