import socket
import re
import urllib2
import os.path
from http_client import HTTPClient
from urlparse import urlparse
import logging

logger = logging.getLogger()

# from http://mattscodecave.com/posts/using-python-and-upnp-to-forward-a-port.html

re_find_location = re.compile(r'location.*?(http://.*?)\s', re.IGNORECASE)
re_find_xml_service = re.compile('<service>.*?</service>', re.IGNORECASE + re.DOTALL)
re_find_xml_service_type = re.compile('<serviceType>.*WANIPConnection.*</serviceType>', re.IGNORECASE)
re_find_xml_control_url = re.compile('<controlURL>(.*?)</controlURL>', re.IGNORECASE)
re_find_external_ip = re.compile('<NewExternalIPAddress>(.*?)</NewExternalIPAddress>')

MY_DIR = os.path.dirname(os.path.realpath(__file__))

SOAP_TEMPLATE = '''<?xml version='1.0' encoding='utf-8'?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
    <SOAP-ENV:Body>
        <m:%(action)s xmlns:m="urn:schemas-upnp-org:service:WANIPConnection:1">
            %(params)s
        </m:%(action)s>
    </SOAP-ENV:Body>
</SOAP-ENV:Envelope>'''


def search_router():
    ssdp_addr = "239.255.255.250"
    ssdp_port = 1900
    ssdp_mx = 2
    ssdp_st = "urn:schemas-upnp-org:device:InternetGatewayDevice:1"
    ssdp_request = "M-SEARCH * HTTP/1.1\r\nHOST: %(ssdp_addr)s:%(ssdp_port)d\r\nMAN: \"ssdp:discover\"\r\nMX: %(ssdp_mx)d\r\nST: %(ssdp_st)s\r\n\r\n" % locals()
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind((get_local_ip(), 1901))
    sock.settimeout(2)
    sock.sendto(ssdp_request, (ssdp_addr, ssdp_port))
    try:
        resp = sock.recv(1000)
    except socket.timeout:
        return None
    locations = re_find_location.findall(resp)
    return locations


def get_control_url(location):
    http_client = HTTPClient()
    try:
        service_desc = http_client.get_page(location)
    except urllib2.URLError:
        return None
    for i in re_find_xml_service.findall(service_desc):
        if re_find_xml_service_type.findall(i):
            control_url = re_find_xml_control_url.findall(i)
            return control_url[0] if control_url else None


def make_request_xml(action, params):
    s_params = ''
    for key in params:
        value = params[key]
        s_params += '<%(key)s>%(value)s</%(key)s>' % locals()
    return SOAP_TEMPLATE % {'action': action, 'params': s_params}


def get_local_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(('10.255.255.255', 1))
        ip = s.getsockname()[0]
    except:
        ip = '127.0.0.1'
    finally:
        s.close()
    return ip


def _get_request_url():
    router_location = search_router()
    if not router_location:
        raise Exception('Not found aviable router')
    router_location = router_location[0]
    logger.debug('UPNP found router %s' % router_location)
    control_url = get_control_url(router_location)
    if not control_url:
        raise Exception('Service WANIPConnection not found')
    logger.debug('UPNP receive control url %s' % control_url)
    router_path = urlparse(router_location)
    request_url = '%s://%s%s' % (router_path[0], router_path[1], control_url)
    return request_url


def add_port_forward(external_port, internal_port, desc):
    request_url = _get_request_url()
    try:
        ip = get_local_ip()
    except:
        raise Exception('Cant determinate local ip')
    logger.debug('UPNP local ip %s' % ip)
    # xml = make_request_port_forward_xml(external_port, internal_port, ip, desc)
    xml = make_request_xml('AddPortMapping', {
        'NewRemoteHost': '0.0.0.0',
        'NewExternalPort': external_port,
        'NewProtocol': 'TCP',
        'NewInternalPort': internal_port,
        'NewInternalClient': ip,
        'NewEnabled': 1,
        'NewPortMappingDescription': desc,
        'NewLeaseDuration': 0,
    })
    http_client = HTTPClient()
    headers = {'SOAPAction': 'urn:schemas-upnp-org:service:WANIPConnection:1#AddPortMapping', 'Content-Type': 'text/xml'}
    return http_client.post_data(request_url, xml, headers=headers)


def del_port_forward(external_port):  # DeletePortMapping
    request_url = _get_request_url()
    try:
        ip = get_local_ip()
    except:
        raise Exception('Cant determinate local ip')
    logger.debug('UPNP local ip %s' % ip)
    # xml = make_request_port_forward_xml(external_port, internal_port, ip, desc)
    xml = make_request_xml('DeletePortMapping', {
        'NewRemoteHost': '0.0.0.0',
        'NewExternalPort': external_port,
        'NewProtocol': 'TCP',
    })
    http_client = HTTPClient()
    headers = {'SOAPAction': 'urn:schemas-upnp-org:service:WANIPConnection:1#DeletePortMapping', 'Content-Type': 'text/xml'}
    return http_client.post_data(request_url, xml, headers=headers)


def get_external_ip():
    request_url = _get_request_url()
    headers = {'SOAPAction': '"urn:schemas-upnp-org:service:WANIPConnection:1#GetExternalIPAddress"', 'Content-Type': 'text/xml'}
    http_client = HTTPClient()
    xml = make_request_xml('GetExternalIPAddress', {})
    resp = http_client.post_data(request_url, xml, headers=headers)
    return re_find_external_ip.findall(resp)[0].strip()


def test1():
    print search_router()


if __name__ == '__main__':
    print get_external_ip()
    #print '-' * 30
    #print add_port_forward(30000, 30000, 'py-upnp-client')
    #print '-' * 30
    #print del_port_forward(30000)
